#ifndef TEXTURE_INCLUDED
#define TEXTURE_INCLUDED

#include <string>
#include <math.h>
#include <ostream>

#include "EPixel.hh"

class TexC
{
public:
  TexC(void);
  TexC(float tX, float tY);
  ~TexC(void);

  inline TexC& operator+=(TexC rhs)
  {
    x += rhs.x;
    y += rhs.y;
    
    return *this;
  }

  float x, y;
};

inline TexC operator-(TexC lhs, const TexC rhs)
{
  return TexC( lhs.x - rhs.x, lhs.y - rhs.y );
}

inline TexC operator+(TexC lhs, const TexC rhs)
{
  return TexC( lhs.x + rhs.x, lhs.y + rhs.y );  
}

inline TexC operator*(float scale, TexC rhs) //vector scale
{
  return TexC( scale*rhs.x, scale*rhs.y );
}

std::ostream& operator<<(std::ostream &out, TexC texC);

class Texture
{
public:
  Texture(void);
  ~Texture(void);

  Texture& operator=(const Texture &rhs);

  inline EPIXEL Lookup(unsigned cX, unsigned cY)
  {
    if( !pPixelData ) return 0;

    if( cX >= texWidth || cY >= texHeight) return 0;

    return pPixelData[cY*texWidth + cX];
  };

  inline EPIXEL Lookup(TexC tC)
  {
    //int iX = rint(tC.x*texWidth);
    //int iY = rint(tC.y*texHeight);

    int iX = (int)( texWidth*(float)tC.x + 0.5);
    int iY = (int)( texHeight*(float)tC.y + 0.5);
    
    if( iX < 0 || iY < 0 ) return 0;

    return Lookup( (unsigned)iX, (unsigned)iY);
  };

  bool FillTexture(const std::string &rFileName);

  unsigned texWidth, texHeight;

  EPIXEL *pPixelData;
};

#endif //TEXTURE_INCLUDED
