#include <math.h>
#include <iostream>
#include <algorithm>
#include <ext/functional>
#include <assert.h>
#include <iomanip>
#include <arpa/inet.h>

#include <iostream>

#include "TvcDriver.hh"

using namespace std;
const unsigned NUM_REGISTERS = 7;
const unsigned STATUS_REGISTER = 6;
const unsigned COMMAND_BUS_ADDRESS_REGISTER = 4;
const unsigned COMMAND_BUS_CONTROL_REGISTER = 5;

//const unsigned long DB_OFFSET = 1000000;

const unsigned long DB_OFFSET = 0x100000; //the location of the depth buffer in the overall framebuffer

TvcDriver::TvcDriver(EPPPort &rTPort) :
  EDriver(),
  localDataRegisters(NUM_REGISTERS),
  remoteDataRegisters(NUM_REGISTERS),
  remoteDataValid(NUM_REGISTERS),
  currentTextureID(0),
  rPort(rTPort)
{
  //generate(localDataRegisters.begin(), localDataRegisters.end(), identity<unsigned char>(0));
  //generate(remoteDataRegisters.begin(), remoteDataRegisters.end(), identity<unsigned char>(0) );
  //generate(remoteDataValid.begin(), remoteDataValid.end(), identity<bool>(false) );
  for(unsigned ii = 0; ii < NUM_REGISTERS; ii++)
    {
      localDataRegisters[ii] = 0;
      remoteDataRegisters[ii] = 0;
      remoteDataValid[ii] = false;
    };

  /*
  for(unsigned long  cVal = 0; cVal < 256; cVal++)
    {
      EPIXEL pixValue = ((unsigned long)cVal)<<EPIXEL_GREEN_OFFSET | ((unsigned long)cVal)<<EPIXEL_BLUE_OFFSET;
      cout << hex << (int)EPixelToHWPixel(pixValue) << dec << endl;
    };

  assert(false);
  */
}

TvcDriver::~TvcDriver(void)
{}

void TvcDriver::DumpRegisters(void)
{
//   for(unsigned ii = 0; ii < 7; ii++)
//     {
//       rPort.WriteAddress(ii);

//       unsigned char value = 0;
//       rPort.ReadData(value);
//       cout << "reg ii: " << ii << " " << setw(2) << setfill('0') << hex << (int)value << dec << endl;
//     };
  unsigned char values[NUM_REGISTERS];
  for(unsigned ii = 0; ii < NUM_REGISTERS; ii++)
    {
      rPort.WriteAddress(ii);

      rPort.ReadData(values[ii]);
      cout << "reg ii: " << ii << " " << setw(2) << setfill('0') << hex << (int)values[ii] << dec << endl;
    };

  unsigned long address = ((unsigned long)values[0])<<16 | ((unsigned long)values[1])<<8 | ((unsigned long)values[2]);
  cout  << setw(8) << setfill('0') << hex << address << dec << " " << address << endl;
}

void TvcDriver::DumpLocalRegisterCache(void)
{
  for(unsigned ii = 0; ii < NUM_REGISTERS; ii++)
    {
     cout << "reg ii: " << ii << " " << setw(2) << setfill('0') << hex << (int)localDataRegisters[ii] << dec << endl;
    };
}

void TvcDriver::ClearRegisters(void)
{
  for(unsigned ii = 0; ii < NUM_REGISTERS-1; ii++) //minus one to not write to the status register
    {
      rPort.WriteAddress(ii);
      rPort.WriteData(0x00);
    };

  DirtyRegisterCache();
}

void TvcDriver::PrintStatus(void)
{
  rPort.WriteAddress(STATUS_REGISTER);

  unsigned char status = 0;
  rPort.ReadData( status );
 
  if( status & 0x80 )
    cout << "epp error bit set" << endl;
  else
    cout << "epp error bit clear" << endl;

  if( status & 0x40 )
     cout << "block reader idle" << endl;
  else
    cout << "block reader busy" << endl;

  if( status & 0x20 )
     cout << "block writer idle" << endl;
  else
    cout << "block writer busy" << endl;

  if( status & 0x10 )
     cout << "pix writer idle" << endl;
  else
    cout << "pix writer busy" << endl;

  if( status & 0x08 )
     cout << "poly scanline idle" << endl;
  else
    cout << "poly scanline busy" << endl;
}

/*
  this was a quicky test case in main that I though might be useful (with changes) to test the 
  framebuffer.  I just stuck the code here where I might find it later.
*/
void TvcDriver::TestRam(void)
{
  const unsigned write_count = 255;
  const unsigned read_count = 255;

  unsigned long bad_count = 0;
  unsigned long presumed_bad_writes = 0;

  for( unsigned ii = 0; ii < write_count; ii++)
    {
      unsigned long dataValue = (unsigned long)rint( (((double)random())/((double)RAND_MAX))* 0xFFFFFFFF);
      cout << "storing value: " << hex << dataValue << dec << endl;
      
      WriteValue(0x00, dataValue);

      unsigned bad_read_count = 0;
      for(unsigned long jj = 0; jj < read_count; jj++)
	{
	  ClearRegisters();
	  
	  unsigned long result = 0;
	  ReadValue(0, result);
	  //driver.DumpRegisters();
	  if( result != dataValue )
	    {
	      cout << setfill('0') << dec << setw(5) << jj
		   << "\t error stored != read stored: " << setfill('0') << setw(8) << hex << dataValue
		   << " result: " << setfill('0') << setw(8) << hex << result << dec << endl;
	      bad_read_count++;
	    };
	};
      bad_count += bad_read_count;

      if(bad_read_count > read_count/2)
	presumed_bad_writes++;
    };

  cout << "bad count: " << bad_count << endl;
  cout << "error factor: " << setprecision(5) << (double)bad_count/(double)(write_count*read_count) << endl;
  cout << "factor that could be due to bad writes: " << setprecision(5)
       << (double)(presumed_bad_writes*write_count)/(double)(write_count*read_count) << endl;
}


void TvcDriver::Blank(unsigned char pixValue)
{
  unsigned long bussValue = HWPixelToBusWidth( pixValue );
 
  const unsigned LEN = 256;
  for(unsigned long ii = 0; ii < 640*480/LEN; ii++)
    SetValue(ii*LEN, LEN, bussValue);

  //for(unsigned long ii = 0; ii < 640*480/4; ii++)
  //WriteValue(ii*4 , value);
}

void TvcDriver::DrawPixel(int xPos, int yPos, unsigned char pixValue)
{
  long address = yPos*640 + xPos;

  unsigned long fbData = 0;
  //unsigned long junk = 0;
  ReadValue(address & 0xFFFFFFFC, fbData);

  switch(address & 0x00000003)
    {
    case 0:
      fbData = ((unsigned long)pixValue)<<24 | (fbData&0x00FFFFFF);
      break;
    case 1:
      fbData = ((unsigned long)pixValue)<<16 | (fbData&0xFF00FFFF);
      break;
    case 2:
      fbData = ((unsigned long)pixValue)<<8 | (fbData&0xFFFF00FF);
      break;
    default: //3
      fbData = ((unsigned long)pixValue) | (fbData&0xFFFFFF00);
      break;
    };

  //hack
  //fbData = HWPixelToBusWidth(pixValue);

  WriteValue(address & 0xFFFFFFFC, fbData);
}

void TvcDriver::DrawLine(int xStart, int yStart, int xEnd, int yEnd,
			 unsigned char pixValue)
{
  int deltaX = xEnd - xStart;
  int deltaY = yEnd - yStart;
  bool xRev = false;
  bool yRev = false;

  if(deltaX < 0)
    {
      xRev = true;
      deltaX *= -1;
    };

  if(deltaY < 0)
    {
      yRev = true;
      deltaY *= -1;
    };

  int lXStart = xStart;
  int lXEnd = xEnd;
  int lYStart = yStart;
  int lYEnd = yEnd;

  if(deltaY > deltaX)
    {
      cout << "y dominant line" << endl;
      if(yRev)
	{
	  lXStart = xEnd;
	  lXEnd = xStart;
	  lYStart = yEnd;
	  lYEnd = yStart;
	  deltaX *= -1;
	};

      for(int cY = lYStart; cY <= lYEnd; cY++)
	{
	  int xVal = lXStart +
	    (int)rint( ((float)deltaX)*((float)(cY - lYStart))/((float)deltaY) );
	  DrawPixel(xVal, cY, pixValue);
	  cout << xVal << " " << cY << endl;
	};
    }else
      {
	cout << "x dominant line" << endl;
	if(xRev)
	  {
	    lXStart = xEnd;
	    lXEnd = xStart;
	    lYStart = yEnd;
	    lYEnd = yStart;
	    deltaY *= -1;
	  };

	for(int cX = lXStart; cX < lXEnd; cX++)
	  {
	    int yVal = lYStart +
	      (int)rint( ((float)deltaY)*((float)(cX - lXStart))/((float)deltaX) );
	    DrawPixel(cX, yVal, pixValue);
	    cout << cX << " " << yVal << endl;
	  };
      };
}

void TvcDriver::ReadValue(unsigned long fbAddress, unsigned long &rValue)
{
  WriteControlBusValue(0x21, fbAddress );

  localDataRegisters[COMMAND_BUS_ADDRESS_REGISTER] = 0x23;
  FlushRegisters();

  rPort.WriteAddress(COMMAND_BUS_CONTROL_REGISTER);
  rPort.WriteData(0xF0); //preform command bus write starting the block reader

  WaitOnBR();

  localDataRegisters[COMMAND_BUS_ADDRESS_REGISTER] = 0x22;
  FlushRegisters();

  /* perform a command bus read */
  rPort.WriteAddress(COMMAND_BUS_CONTROL_REGISTER);
  rPort.WriteData(0x0F);

  /* read values from command bus registers*/

  rPort.WriteAddress(0);
  unsigned char byte3 = 0;
  rPort.ReadData(byte3);

  rPort.WriteAddress(1);
  unsigned char byte2 = 0;
  rPort.ReadData(byte2);

  rPort.WriteAddress(2);
  unsigned char byte1 = 0;
  rPort.ReadData(byte1);

  rPort.WriteAddress(3);
  unsigned char byte0 = 0;
  rPort.ReadData(byte0);

  DirtyRegisterCache(); //leave things sane for the next command

  rValue = ((unsigned long)byte3)<<24 | ((unsigned long)byte2)<<16 |
    ((unsigned long)byte1)<<8 | (unsigned long)byte0;
}

void TvcDriver::WriteValue(unsigned long fbAddress, unsigned long newValue)
{
  WriteControlBusValue(0x11, fbAddress);
  WriteControlBusValue(0x12, newValue );

  localDataRegisters[COMMAND_BUS_ADDRESS_REGISTER] = 0x13;
  FlushRegisters();

  rPort.WriteAddress(COMMAND_BUS_CONTROL_REGISTER);
  rPort.WriteData(0xF0);

  WaitOnBW();
}

void TvcDriver::SetValue(unsigned long fbAddress, unsigned long numBytes, unsigned long newValue)
{
  WriteControlBusValue(0x01, fbAddress);
  WriteControlBusValue(0x02, numBytes );
  WriteControlBusValue(0x03, newValue);

  localDataRegisters[COMMAND_BUS_ADDRESS_REGISTER] = 0x04;
  FlushRegisters();

  rPort.WriteAddress(COMMAND_BUS_CONTROL_REGISTER);
  rPort.WriteData(0xF0);
 
  //DumpLocalRegisterCache();

  WaitOnBS();
}

// ***************** start EDriver overloads ******************
void TvcDriver::PolyScanLine(unsigned sX, long startZ, unsigned eX, long endZ, unsigned yValue, TexC texStart, TexC texInc, EPIXEL pixValue)
{
  /*
  cout << "TVCDriver::PolyScanLine sX: " << sX << " startZ: " << startZ 
       << " eX: " << eX << " endZ: " << endZ << " yValue: " << yValue << endl;
  */
  unsigned long fbAddress = yValue*640 + sX;
  unsigned long dbAddress = yValue*640*4 + sX*4 + DB_OFFSET;

  long zIncrement = 0;
  if( eX != sX )
    zIncrement = (endZ - startZ)/((long)(eX - sX));

  unsigned numPixels = eX - sX;

  WriteControlBusValue(0xf0, fbAddress);
  WriteControlBusValue(0xf1, dbAddress);
  WriteControlBusValue(0xf2, startZ);
  WriteControlBusValue(0xf3, zIncrement);

  WriteControlBusValue(0xf4, numPixels); //only lowest 16 bits are used...
  WriteControlBusValue(0xf5, (int)EPixelToHWPixel(pixValue) ); //only lowest 8 bits are used.

  if( currentTextureID != 0 )
    {
      long tc_x_start = rint(texStart.x*65535.00);
      long tc_y_start = rint(texStart.y*65535.00);
      long tc_x_inc   = rint(texInc.x*65535.00);
      long tc_y_inc   = rint(texInc.y*65535.00);
      WriteControlBusValue(0xf6, (tc_x_start<<16)&0xFFFF0000 | tc_y_start&0x0000FFFF);
      WriteControlBusValue(0xf7, (tc_x_inc<<16)&0xFFFF0000   | tc_y_inc&0x0000FFFF);
    };

  WriteControlBusValue(0xf8, currentTextureID);

  /*
  localDataRegisters[17] = numPixels&0x000000FF;
  localDataRegisters[16] = (numPixels&0x0000FF00)>>8;
  localDataRegisters[18] = (int)EPixelToHWPixel(pixValue);
  */

  localDataRegisters[COMMAND_BUS_ADDRESS_REGISTER] = 0xF9;
  FlushRegisters();

  rPort.WriteAddress(COMMAND_BUS_CONTROL_REGISTER);
  rPort.WriteData(0xF0);

  //DumpLocalRegisterCache();

  WaitOnPS();
}

void TvcDriver::ClearFB(EPIXEL pixValue)
{
  unsigned long bussValue = HWPixelToBusWidth( EPixelToHWPixel(pixValue) );

  const unsigned LEN = 256;
  for(unsigned long ii = 0; ii < 640*480/LEN; ii++)
    SetValue(ii*LEN, LEN, bussValue);
}

void TvcDriver::ClearDB(long depthVal)
{
  const unsigned LEN = 256;
  for(unsigned long ii = 0; ii < 640*480*4/LEN; ii++)
    SetValue(DB_OFFSET+(ii*LEN), LEN, depthVal);

  /*
  for(unsigned long ii = 0; ii < 640*480; ii++)
  WriteValue(ii*4+DB_OFFSET , depthVal);
  */
}

void TvcDriver::Flush(void)
{}

void TvcDriver::UpLoadTexture(Texture &rTexture, unsigned textureID)
{
  unsigned long texAddress = 0;
  switch(textureID)
    {
    case 1:
      texAddress = 0x300000;
      break;
    case 2:
      texAddress = 0x380000;
      break;
    case 3:
      texAddress = 0x400000;
      break;
    case 4:
      texAddress = 0x480000;
      break;
    case 5:
      texAddress = 0x500000;
      break;
    case 6:
      texAddress = 0x580000;
      break;
    case 7:
      texAddress = 0x600000;
      break;
    default:
      cerr << "invalid texture id mentioned in TvcDriver::UpLoadTexture. aborting." << endl;
      return;
    };

  if( !rTexture.pPixelData )
    {
      cerr << "TvcDriver::UpLoadTexture texture data seems to be missing! aborting." << endl;
      return;
    };

  if( rTexture.texWidth != 256 || rTexture.texHeight != 256)
    {
      cerr << "TvcDriver::UpLoadTexture texture has invalid size. aborting." << endl;
      return;
    };

  cout << "TvcDriver::UpLoadTexture " << textureID << " starting..." << endl;
  for(unsigned ii = 0; ii < rTexture.texHeight; ii++)
    for(unsigned jj = 0; jj < rTexture.texWidth; jj++)
      {
	EPIXEL pixValue = rTexture.Lookup(jj, ii);

	WriteValue(texAddress, pixValue);
	texAddress += 4;
      };
  cout << "TvcDriver::UpLoadTexture " << textureID << " done." << endl;
}

void TvcDriver::SetCTexture(unsigned cTextureID)
{
  currentTextureID = cTextureID;
}

// ****************** end EDriver overloads *******************

unsigned char TvcDriver::EPixelToHWPixel(EPIXEL pixValue)
{
  //a very painful pixel conversion
  unsigned char redComp   = (unsigned char)( ( (pixValue & EPIXEL_RED_MASK)>>EPIXEL_RED_OFFSET ) & 0xC0 );
  unsigned char greenComp = (unsigned char)( (( (pixValue & EPIXEL_GREEN_MASK)>>EPIXEL_GREEN_OFFSET ) & 0xE0) >> 2 );
  unsigned char blueComp  = (unsigned char)( (( (pixValue & EPIXEL_BLUE_MASK)>>EPIXEL_BLUE_OFFSET   ) & 0xE0) >> 5 );
  return redComp | greenComp | blueComp;
}

unsigned long TvcDriver::HWPixelToBusWidth(unsigned char hwPixel)
{
  return ((unsigned long)hwPixel)<<24 | ((unsigned long)hwPixel)<<16 | ((unsigned long)hwPixel)<<8 | ((unsigned long)hwPixel);
}

void TvcDriver::WriteControlBusValue(unsigned char addr, unsigned long value)
{
  Write32(value, 0);
  localDataRegisters[COMMAND_BUS_ADDRESS_REGISTER] = addr;
  FlushRegisters();

  rPort.WriteAddress(COMMAND_BUS_CONTROL_REGISTER);
  rPort.WriteData(0xF0);
}

void TvcDriver::DirtyRegisterCache(void)
{
  for(unsigned ii = 0; ii < NUM_REGISTERS; ii++)
    remoteDataValid[ii] = false;
}

void TvcDriver::FlushRegisters(void)
{
  unsigned char tvcIntAddress = 0xFF; // an invalid value
  for(unsigned ii = 0; ii < NUM_REGISTERS; ii++)
    {
      if( !remoteDataValid[ii] || ( localDataRegisters[ii] != remoteDataRegisters[ii] ) )
	{
	  if( ii != COMMAND_BUS_CONTROL_REGISTER )
	    {
	      if( tvcIntAddress != ii )
		{
		  rPort.WriteAddress(ii);
		  tvcIntAddress = ii;
		};

	      rPort.WriteData( localDataRegisters[ii] );
	      remoteDataRegisters[ii] = localDataRegisters[ii];
	      remoteDataValid[ii] = true;
	      tvcIntAddress++;
	    };
	};
    };
}

void TvcDriver::Write32(unsigned long value, unsigned regOffset)
{
  localDataRegisters[regOffset + 0] = static_cast<unsigned char>( (value & 0xFF000000)>>24 );
  localDataRegisters[regOffset + 1] = static_cast<unsigned char>( (value & 0x00FF0000)>>16 );
  localDataRegisters[regOffset + 2] = static_cast<unsigned char>( (value & 0x0000FF00)>>8 );;
  localDataRegisters[regOffset + 3] = static_cast<unsigned char>( (value & 0x000000FF) );
}

void TvcDriver::WaitOnBW(void)
{
  rPort.WriteAddress(STATUS_REGISTER);

  for(unsigned ii = 0; ii < 1000; ii++)
    {
      unsigned char status = 0;
      rPort.ReadData( status );
      if( status & 0x20 ) return;
    }

  cerr << "timeout in TvcDriver::WaitOnBW" << endl;
  assert(false);
}

void TvcDriver::WaitOnBR(void)
{
  rPort.WriteAddress(STATUS_REGISTER);

  for(unsigned ii = 0; ii < 1000; ii++)
    {
      unsigned char status = 0;
      rPort.ReadData( status );
      if( status & 0x40 ) return;
    }

  cerr << "timeout in TvcDriver::WaitOnBR" << endl;
  assert(false);
}

void TvcDriver::WaitOnBS(void)
{
  rPort.WriteAddress(STATUS_REGISTER);

  for(unsigned ii = 0; ii < 1000; ii++)
    {
      unsigned char status = 0;
      rPort.ReadData( status );
      if( status & 0x10 ) return;
    }

  cerr << "timeout in TvcDriver::WaitOnBS" << endl;
  assert(false);
}

void TvcDriver::WaitOnPS(void)
{
  rPort.WriteAddress(STATUS_REGISTER);

  for(unsigned ii = 0; ii < 1000000; ii++)
    {
      unsigned char status = 0;
      rPort.ReadData( status );
      if( status & 0x08 ) return;
    }

  cerr << "timeout in TvcDriver::WaitOnPS" << endl;
  assert(false);
}
