#include <iostream>
#include <assert.h>

#include "XEDriver.hh"

using namespace std;

XEDriver::XEDriver(void) :
  EDriver(), xImgWindow(640,480), currentTex(0),
  frameBuffer(640, 480), depthBuffer(640,480)
{}

XEDriver::~XEDriver(void)
{}

void XEDriver::CreateRenderContext(void)
{
  if( !xImgWindow.CreateAll() )
    {
      cerr << "error opening x" << endl;
      assert(false);
    };
  
  frameBuffer.Initialize();
  depthBuffer.Initialize();
}

void XEDriver::DeleteRenderContext(void)
{
  xImgWindow.DeleteAll();
}

void XEDriver::PolyScanLine(unsigned sX, long sZ, unsigned eX, long eZ, unsigned yVal,
			    TexC texStart, TexC texInc, EPIXEL pixValueIn)
{
  long deltaZ = eZ - sZ;
  long mZ = 0;
  if(sX != eX)
    mZ = deltaZ/((long)((eX - sX)));

  long cZ = sZ;
  TexC cTex(texStart);

  if( eX > 639 ) eX = 639;

  if( currentTex )
    {
      Texture &rTex = texArray[currentTex-1];
      for(unsigned ii = sX; ii <= eX; ii++)
	{
	  if( cZ > (depthBuffer.Pixel(ii, yVal)).value )
	    {
	      depthBuffer.Pixel(ii, yVal) = DepthPixel( cZ );
	      frameBuffer.Pixel(ii, yVal) = rTex.Lookup(cTex);
	    };
	  
	  cTex += texInc;
	  cZ += mZ;
	};
    }else
    for(unsigned ii = sX; ii <= eX; ii++)
      {
	if( cZ > (depthBuffer.Pixel(ii, yVal)).value )
	  {
	    depthBuffer.Pixel(ii, yVal) = DepthPixel( cZ );
	    frameBuffer.Pixel(ii, yVal) = pixValueIn;
	  };
	
	cZ += mZ;
      };

  /*
  for(unsigned ii = sX; ii <= eX; ii++)
    {
      float cDepth = sZ;
      if(ii != sX )
	cDepth += ( ((float)(eZ - sZ))/( (float)(ii - sX) ) );
      
      if( cDepth < (depthBuffer.Pixel(ii, yVal)).value ) continue;

      depthBuffer.Pixel(ii, yVal) = DepthPixel( (long)rint(cDepth) );

      frameBuffer.Pixel(ii, yVal) = pixValue;
    };
  */
}

void XEDriver::ClearFB(EPIXEL pixValue)
{
  frameBuffer.Clear( pixValue );
}

void XEDriver::ClearDB(long depthVal)
{
  depthBuffer.Clear( depthVal );
}

void XEDriver::Flush(void)
{
  //cout << "XEDriver::Flush" << endl;
  xImgWindow.UpdateWindow(frameBuffer);
}

void XEDriver::UpLoadTexture(Texture &rTexture, unsigned textureID)
{
  if( textureID == 0 || textureID > 8 ) return;

  texArray[textureID-1] = rTexture;
}

void XEDriver::SetCTexture(unsigned cTextureID)
{
  currentTex = cTextureID;
  /* a qnd test to see if texture data is ok
  if(currentTex == 0 || currentTex > 8) return;

  ClearFB(0);

  Texture &rTexture = texArray[currentTex - 1];
  for(unsigned cY = 0; cY < rTexture.texHeight; cY++)
    for(unsigned cX = 0; cX < rTexture.texWidth; cX++)
      {
	EPIXEL cPix = rTexture.Lookup(cX, cY);
	frameBuffer.Pixel(cX, cY) = cPix;
      };

  xImgWindow.UpdateWindow(frameBuffer);
  */
}
