library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity block_reader is
	Port(
   --memory user access bus 
   mu_bus_address 	   : in  std_logic_vector(11 downto 0);
   mu_bus_data		   : inout std_logic_vector(31 downto 0);
   mu_bus_write	   	: in  std_logic; --if high the MCU is going to write to a mu
   --if high the MCU expects the selected MU to present its data on the mu_bus_data or to
   --copy data from the mu_bus_data into its internal cache/registers.
   mu_bus_strobe			: in  std_logic;
   
   block_reader_irq		: out std_logic;
   block_reader_select	: in  std_logic;

   --blah		 
   tvc_clock		      : in	std_logic;
   tvc_reset		      : in 	std_logic;

   --interface to the eppcu
   cmd_bus_data      : in std_logic_vector(31 downto 0);
   cmd_bus_addr      : in std_logic_vector(7 downto 0);
   cmd_bus_strobe    : in std_logic;
   cmd_bus_data_out  : out std_logic_vector(31 downto 0);
   
    epp_idle			      : out std_logic
   );
end block_reader;

architecture Behavioral of block_reader is
	signal fb_address_reg 	: STD_LOGIC_VECTOR(31 downto 0) := (others => '0');
	--signal fb_data_reg		: STD_LOGIC_VECTOR(31 downto 0);

	signal selected_once    : STD_LOGIC := '0';

	TYPE PR_INT_STATE is ( PR_IDLE, PR_MCU_DRIVEN);
	signal pr_state         : PR_INT_STATE := PR_IDLE;

	signal mu_bus_data_in  : std_logic_vector(31 downto 0);
	signal mu_bus_data_out : std_logic_vector(31 downto 0);

begin
	process (pr_state)
	begin
		if pr_state = PR_IDLE then
			epp_idle <= '1';
		else
			epp_idle <= '0';
		end if;
	end process;

	mu_bus_data_in <= mu_bus_data;
	process(mu_bus_data_out, block_reader_select, mu_bus_strobe, mu_bus_write)
	begin
		if block_reader_select = '1' and mu_bus_strobe = '1' and mu_bus_write = '0' then
			mu_bus_data <= mu_bus_data_out;
		else
			mu_bus_data <= (others => 'Z');
		end if;
	end process;

	Process (tvc_clock)
	begin
	if tvc_clock = '1' and tvc_clock'event then
	if tvc_reset = '1' then
		block_reader_irq <= '0';
		pr_state <= PR_IDLE;
	else --not reset
	   case pr_state is
	   when PR_IDLE =>
	   	if cmd_bus_strobe = '1' then
            case cmd_bus_addr is
            when X"21" =>
               fb_address_reg <=  cmd_bus_data(31 downto 0);
            when X"22" =>
               block_reader_irq <= '1';
               pr_state <= PR_MCU_DRIVEN;
               selected_once <= '0';
            when others =>
               null;
            end case;
			end if;
		when PR_MCU_DRIVEN =>
			if block_reader_select = '0' and selected_once = '1' then
				pr_state <= PR_IDLE;
				block_reader_irq <= '0';
			elsif block_reader_select = '1' then
				selected_once <= '1';
				if mu_bus_address = X"FFC" and mu_bus_write = '0' and mu_bus_strobe = '1' then
					mu_bus_data_out <= X"00" & X"000004";
				elsif mu_bus_address = X"FFD" and mu_bus_write = '0' and mu_bus_strobe = '1' then
					mu_bus_data_out <= fb_address_reg;
				elsif mu_bus_address = X"000" and mu_bus_write = '1' and mu_bus_strobe = '1' then
					cmd_bus_data_out <= mu_bus_data_in;
				end if;
			end if;
		end case;
	end if; --reset
	end if; --clock
	end process;
end Behavioral;
