library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity block_writer is
	Port(
	--memory user access bus 
	mu_bus_address 	   : in     std_logic_vector(11 downto 0);
	mu_bus_data		   : inout  std_logic_vector(31 downto 0);
	mu_bus_write		   : in     std_logic; --if high the MCU is going to write to a mu
	--if high the MCU expects the selected MU to present its data on the mu_bus_data or to
	--copy data from the mu_bus_data into its internal cache/registers.
	mu_bus_strobe			: in     std_logic;
	
	block_writer_irq		: out    std_logic;
	block_writer_select	: in     std_logic;

	--blah		 
	tvc_clock		      : in		std_logic;
	tvc_reset		      : in  	std_logic;

	--interface to the eppcu
   cmd_bus_data      : in std_logic_vector(31 downto 0);
   cmd_bus_addr      : in std_logic_vector(7 downto 0);
   cmd_bus_strobe    : in std_logic;

	epp_idle			      : out    std_logic
	);
end block_writer;

architecture Behavioral of block_writer is
	signal fb_address_reg 	: STD_LOGIC_VECTOR(31 downto 0) := (others => '0');
	signal fb_data_reg		: STD_LOGIC_VECTOR(31 downto 0) := (others => '0');

	signal selected_once : STD_LOGIC := '0';

	TYPE BW_INT_STATE is ( BW_IDLE, BW_MCU_DRIVEN, BW_COMPLETING );
	signal bw_state : BW_INT_STATE := BW_IDLE;

 	signal mu_bus_data_out : std_logic_vector(31 downto 0) := (others => '0');

	procedure HandleMCUDrivenClock( signal bw_state	: inout BW_INT_STATE;
								signal fb_address_reg 		: in STD_LOGIC_VECTOR(31 downto 0);
								signal fb_data_reg			: in	STD_LOGIC_VECTOR(31 downto 0);
								signal selected_once			: inout std_logic;
								signal mu_bus_address 		: in std_logic_vector(11 downto 0);
								signal mu_bus_data_out		: out std_logic_vector(31 downto 0);
								signal mu_bus_write			: in  std_logic;
								signal mu_bus_strobe		: in  std_logic;
								signal block_writer_irq		: out std_logic;
								signal block_writer_select : in std_logic) is
	begin
		if block_writer_select = '0' and selected_once = '1' then
			bw_state <= BW_COMPLETING;
			block_writer_irq <= '0';
		 elsif block_writer_select = '1' then
		 	selected_once <= '1';
			if mu_bus_address = X"FFC" and mu_bus_write = '0' and mu_bus_strobe = '1' then
				mu_bus_data_out <= X"80" & X"000004";
			elsif mu_bus_address = X"FFD" and mu_bus_write = '0' and mu_bus_strobe = '1' then
				mu_bus_data_out <= fb_address_reg;
			elsif mu_bus_address = X"000" and mu_bus_write = '0' and mu_bus_strobe = '1' then
				mu_bus_data_out <= fb_data_reg;
			end if;
		end if;
	end HandleMCUDrivenClock;

begin
	process(bw_state)
	begin
		if bw_state = BW_IDLE then
			epp_idle <= '1';
		else
			epp_idle <= '0';
		end if;
	end process;

	process(mu_bus_data_out, block_writer_select, mu_bus_strobe, mu_bus_write)
	begin
		if block_writer_select = '1' and mu_bus_strobe = '1' and mu_bus_write = '0' then
			mu_bus_data <= mu_bus_data_out;
		else
			mu_bus_data <= (others => 'Z');
		end if;
	end process;

	Process (tvc_clock)
	begin
	if tvc_clock = '1' and tvc_clock'event then
	if tvc_reset = '1' then
		--fb_address_reg <= X"0000_0000";
		--fb_data_reg <= X"0000_0000";
		BW_state <= BW_IDLE;
		block_writer_irq <= '0';
	else --not reset
		case BW_state is
	  	when BW_IDLE =>
		  	if cmd_bus_strobe = '1' then
            case cmd_bus_addr is
            when X"11" =>
               fb_address_reg <=  cmd_bus_data(31 downto 0);
            when X"12" =>
               fb_data_reg	<= 	cmd_bus_data(31 downto 0);
            when X"13" =>
               --fb_data_reg <= X"1234_5678";
               block_writer_irq <= '1';
               BW_state <= BW_MCU_DRIVEN;
               selected_once <= '0';
            when others =>
               null;
            end case;
			end if;
		when BW_MCU_DRIVEN =>
			HandleMCUDrivenClock(bw_state, fb_address_reg, fb_data_reg, selected_once,
							mu_bus_address, mu_bus_data_out, mu_bus_write, mu_bus_strobe, block_writer_irq, block_writer_select);
		when BW_COMPLETING =>
			BW_state <= BW_IDLE;
		end case;
	end if; --reset
	end if; --clock
	end process;
end Behavioral;
