----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    22:36:41 08/04/2006 
-- Design Name: 
-- Module Name:    mcu_sched - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity mcu_sched is
   Port (
		tvc_clock		: in	std_logic;
		tvc_reset		: in 	std_logic;

		mu_bus_address 	: out std_logic_vector(11 downto 0);
		mu_bus_write		: out  std_logic;
		mu_bus_strobe		: out  std_logic;

		mcu_hl_start		: out std_logic;
		mcu_hl_busy			: in std_logic;

		--memory users (mu's) irq/select lines
		odd_sc_irq		: in std_logic;
		odd_sc_select		: out std_logic;

		even_sc_irq		: in std_logic;
		even_sc_select		: out std_logic;

		block_reader_irq		: in std_logic;
		block_reader_select	: out std_logic;

		block_writer_irq		: in std_logic;
		block_writer_select	: out std_logic;				 

		rp_writer_irq		: in std_logic;
		rp_writer_select	: out std_logic;

		line_drawer_irq	: in std_logic;
		line_drawer_select	: out std_logic;
		
		poly_scanline_irq	: in std_logic;
		poly_scanline_select : out std_logic
	);
end mcu_sched;

architecture Behavioral of mcu_sched is

	TYPE MCU_SCHED_STATE is (MCU_SCHED_IDLE, MCU_SCHED_WAITSTATE, MCU_SCHED_WAITING, MCU_SCHED_END_WAITSTATE, MCU_SCHED_END_WAITSTATE2);
	signal sched_state : MCU_SCHED_STATE := MCU_SCHED_IDLE;

	--0 is none 1 is the odd_sc 2 is the even_sc, 3 is the block_reader 4 is the block_writer
	--5 is the rp_writer 6 is the line drawer 7 is the poly_scanline
	signal active_mu : STD_LOGIC_VECTOR(2 downto 0) := (others => '0');

	--we service mus in a round-robbin fashion with the exception of the pixel-pusher(s)' scanline caches that have priority
	--this should not ever be 0 or 1, as these are the scanline caches
	--this is the next memory user that should get priority if it and another memory user wants a transfer.
	signal next_mu_to_check : STD_LOGIC_VECTOR(2 downto 0) := "011";
	
begin

	process(tvc_clock)
	variable starting_xfer :	boolean;
	begin
	if tvc_clock = '1' and tvc_clock'event then
	if tvc_reset = '1' then
		sched_state <= MCU_SCHED_IDLE;
		active_mu <= "000";
		next_mu_to_check <= "011";

		mcu_hl_start <= '0';
		
		mu_bus_address <= X"000";
		mu_bus_write	<= '0';
		mu_bus_strobe	<= '0';

		odd_sc_select <= '0';
		even_sc_select <= '0';
		block_reader_select <= '0';
		block_writer_select <= '0';
		rp_writer_select <= '0';
		line_drawer_select <= '0';
		poly_scanline_select <= '0';
	else --not reset
		starting_xfer := false;
		case sched_state is
		when MCU_SCHED_IDLE =>
			 	if odd_sc_irq = '1' then
					active_mu <= "001";
					odd_sc_select <= '1';
					starting_xfer := true;
				elsif even_sc_irq = '1' then
					active_mu <= "010";
					even_sc_select <= '1';
					starting_xfer := true;
				else 
					case next_mu_to_check is
					when "011" =>
						if block_reader_irq = '1' then
							active_mu <= "011";
							block_reader_select <= '1';
							next_mu_to_check <= "100";
							starting_xfer := true;
						elsif block_writer_irq = '1' then
							active_mu <= "100";
							block_writer_select <= '1';
							next_mu_to_check <= "101";
							starting_xfer := true;
						elsif rp_writer_irq = '1' then
							active_mu <= "101";
							rp_writer_select <= '1';
							next_mu_to_check <= "110";
							starting_xfer := true;
						elsif line_drawer_irq = '1' then
							active_mu <= "110";
							line_drawer_select <= '1';
							next_mu_to_check <= "111";
							starting_xfer := true;
						elsif poly_scanline_irq	= '1' then
							active_mu <= "111";
							poly_scanline_select <= '1';
							next_mu_to_check <= "011";
							starting_xfer := true;
						end if;
					when "100" =>
						if block_writer_irq = '1' then
							active_mu <= "100";
							block_writer_select <= '1';
							next_mu_to_check <= "101";
							starting_xfer := true;
						elsif rp_writer_irq = '1' then
							active_mu <= "101";
							rp_writer_select <= '1';
							next_mu_to_check <= "110";
							starting_xfer := true;
						elsif line_drawer_irq = '1' then
							active_mu <= "110";
							line_drawer_select <= '1';
							next_mu_to_check <= "111";
							starting_xfer := true;
						elsif poly_scanline_irq	= '1' then
							active_mu <= "111";
							poly_scanline_select <= '1';
							next_mu_to_check <= "011";
							starting_xfer := true;
						elsif block_reader_irq = '1' then
							active_mu <= "011";
							block_reader_select <= '1';
							next_mu_to_check <= "100";
							starting_xfer := true;
						end if;
					when "101" =>
						if rp_writer_irq = '1' then
							active_mu <= "101";
							rp_writer_select <= '1';
							next_mu_to_check <= "110";
							starting_xfer := true;
						elsif line_drawer_irq = '1' then
							active_mu <= "110";
							line_drawer_select <= '1';
							next_mu_to_check <= "111";
							starting_xfer := true;
						elsif poly_scanline_irq	= '1' then
							active_mu <= "111";
							poly_scanline_select <= '1';
							next_mu_to_check <= "011";
							starting_xfer := true;
						elsif block_reader_irq = '1' then
							active_mu <= "011";
							block_reader_select <= '1';
							next_mu_to_check <= "100";
							starting_xfer := true;
						elsif block_writer_irq = '1' then
							active_mu <= "100";
							block_writer_select <= '1';
							next_mu_to_check <= "101";
							starting_xfer := true;
						end if;
					when "110" =>
						if line_drawer_irq = '1' then
							active_mu <= "110";
							line_drawer_select <= '1';
							next_mu_to_check <= "111";
							starting_xfer := true;
						elsif poly_scanline_irq	= '1' then
							active_mu <= "111";
							poly_scanline_select <= '1';
							next_mu_to_check <= "011";
							starting_xfer := true;
						elsif block_reader_irq = '1' then
							active_mu <= "011";
							block_reader_select <= '1';
							next_mu_to_check <= "100";
							starting_xfer := true;
						elsif block_writer_irq = '1' then
							active_mu <= "100";
							block_writer_select <= '1';
							next_mu_to_check <= "101";
							starting_xfer := true;
						elsif rp_writer_irq = '1' then
							active_mu <= "101";
							rp_writer_select <= '1';
							next_mu_to_check <= "110";
							starting_xfer := true;
						end if;
					when "111" =>
						if poly_scanline_irq	= '1' then
							active_mu <= "111";
							poly_scanline_select <= '1';
							next_mu_to_check <= "011";
							starting_xfer := true;
						elsif block_reader_irq = '1' then
							active_mu <= "011";
							block_reader_select <= '1';
							next_mu_to_check <= "100";
							starting_xfer := true;
						elsif block_writer_irq = '1' then
							active_mu <= "100";
							block_writer_select <= '1';
							next_mu_to_check <= "101";
							starting_xfer := true;
						elsif rp_writer_irq = '1' then
							active_mu <= "101";
							rp_writer_select <= '1';
							next_mu_to_check <= "110";
							starting_xfer := true;
						elsif line_drawer_irq = '1' then
							active_mu <= "110";
							line_drawer_select <= '1';
							next_mu_to_check <= "111";
							starting_xfer := true;
						end if;
					when others => -- a bug
						next_mu_to_check <= "011";
					end case;
			end if;
			if starting_xfer = true then
				sched_state <= MCU_SCHED_WAITSTATE;
				mu_bus_address <= X"FFC";
				mu_bus_strobe <= '1';
				mcu_hl_start <= '1';
				mu_bus_write <= '0'; --we want to read an address
			end if;
		when MCU_SCHED_WAITSTATE =>
			sched_state <= MCU_SCHED_WAITING;
			--mu_bus_strobe <= '0'; --dont change this now and get a glitch.
			mcu_hl_start <= '0';
		when MCU_SCHED_WAITING =>
			mu_bus_strobe <= '0';
			if mcu_hl_busy = '0' then
				case active_mu is
				when "001" => odd_sc_select <= '0';
				when "010" => even_sc_select <= '0';
				when "011" => block_reader_select <= '0';
				when "100" => block_writer_select <= '0';
				when "101" => rp_writer_select <= '0';
				when "110" => line_drawer_select <= '0';
				when "111" => poly_scanline_select <= '0';
				when others => null;
				end case;
				active_mu <= "000";
				sched_state <= MCU_SCHED_END_WAITSTATE;
			end if;
		when MCU_SCHED_END_WAITSTATE =>
			sched_state <= MCU_SCHED_END_WAITSTATE2;
		when MCU_SCHED_END_WAITSTATE2 =>
			sched_state <= MCU_SCHED_IDLE;
		end case;
	end if;
	end if;
	end process;
end Behavioral;

