----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    22:59:02 08/04/2006 
-- Design Name: 
-- Module Name:    mcu_struct - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

-- the memory controller unit (mcu) manages the dram. Because the chip will perpeturarly
-- be bandwidth starved, it will basiclly set the timing on everything.
-- memory users (mu) will assert that they need reading/writing done on their behalf, and the mcu
-- will preform the desired operations when it can.  the data for reading/writing will be held
-- in the 'calling' mu.  the calling mu's will share an access buss to the mcu.

entity mcu_struct is
    Port (
		dram_d_buss		: inout std_logic_vector(31 downto 0);
		dram_a_buss 	: out std_logic_vector(11 downto 0);
		dram_ras_0 		: out std_logic;  -- all the rest of the dram signals are active low
		dram_ras_1 		: out std_logic;
		dram_ras_2 		: out std_logic;
		dram_ras_3 		: out std_logic;
		dram_cas_0 		: out std_logic;
		dram_cas_1 		: out std_logic;
		dram_cas_2 		: out std_logic;
		dram_cas_3 		: out std_logic;
		dram_we			: out std_logic;
		 
		--the mcu's locic interface to the rest of the tvc		 
		tvc_clock		: in	std_logic;
		tvc_reset		: in 	std_logic;

 		--memory user access buss 
		mu_bus_address 	: out std_logic_vector(11 downto 0);
		mu_bus_data		: inout std_logic_vector(31 downto 0);
		mu_bus_write		: out  std_logic; --if high the MCU is going to write to a mu
		--if high the MCU expects the selected MU to present its data on the mu_bus_data or to
		--copy data from the mu_bus_data into its internal cache/registers.
		mu_bus_strobe			: out  std_logic;

		odd_sc_irq				: in std_logic;
		odd_sc_select			: out std_logic;
		even_sc_irq				: in std_logic;
		even_sc_select			: out std_logic;
		block_reader_irq		: in std_logic;
		block_reader_select	: out std_logic;
		block_writer_irq		: in std_logic;
		block_writer_select	: out std_logic;				 
		rp_writer_irq			: in std_logic;
		rp_writer_select		: out std_logic;
		line_drawer_irq		: in std_logic;
		line_drawer_select	: out std_logic;
		poly_scanline_irq		: in std_logic;
		poly_scanline_select : out std_logic
		);
		
end mcu_struct;

architecture Behavioral of mcu_struct is

	signal sched_mu_bus_address : std_logic_vector(11 downto 0);
	signal sched_mu_bus_write : std_logic;
	signal sched_mu_bus_strobe : std_logic;

	component mcu_sched is
   Port (
		tvc_clock				: in	std_logic;
		tvc_reset				: in 	std_logic;
		mu_bus_address 		: out std_logic_vector(11 downto 0);
		mu_bus_write			: out  std_logic;
		mu_bus_strobe			: out  std_logic;
		mcu_hl_start			: out std_logic;
		mcu_hl_busy				: in std_logic;
		odd_sc_irq				: in std_logic;
		odd_sc_select			: out std_logic;
		even_sc_irq				: in std_logic;
		even_sc_select			: out std_logic;
		block_reader_irq		: in std_logic;
		block_reader_select	: out std_logic;
		block_writer_irq		: in std_logic;
		block_writer_select	: out std_logic;				 
		rp_writer_irq			: in std_logic;
		rp_writer_select		: out std_logic;
		line_drawer_irq		: in std_logic;
		line_drawer_select	: out std_logic;
		poly_scanline_irq		: in std_logic;
		poly_scanline_select : out std_logic
	);
	end component;
	
	signal mcu_hl_mu_bus_address : std_logic_vector(11 downto 0);
	--signal mcu_hl_mu_bus_data		: std_logic_vector(31 downto 0);
	signal mcu_hl_mu_bus_write	: std_logic;
	signal mcu_hl_mu_bus_strobe	: std_logic;

	component mcu_hl is
   Port (
		mu_bus_address 				: out std_logic_vector(11 downto 0);
		mu_bus_data					: inout std_logic_vector(31 downto 0);
		mu_bus_write					: out  std_logic;
		mu_bus_strobe					: out  std_logic;
		tvc_clock						: in		std_logic;
		tvc_reset						: in 	std_logic;
		mcu_hl_start_strobe			: in std_logic;
		mcu_hl_busy						: out std_logic;
		mcu_ll_data_out 				: out std_logic_vector(31 downto 0);
		mcu_ll_data_in					: in std_logic_vector(31 downto 0);
		mcu_ll_address					: out std_logic_vector(31 downto 2);
		mcu_ll_xfer_write				: out	std_logic;
		mcu_ll_xfer_count				: out std_logic_vector(11 downto 0);
		mcu_ll_data_valid_done 		: in std_logic; --either the mcu is done reading our data or has just written new data to us. high only a single cycle
		mcu_ll_start_strobe 			: out std_logic;
		mcu_ll_idle						: in std_logic --goes high on cycle after start strobe. goes low if transfer complete or an error occurs.
		);
	end component;

	--these signals interconnect the mcu_sched and the mcu_hl
	signal mcu_sched_hl_start			: std_logic;
	signal mcu_sched_hl_busy			: std_logic; --also used for the mu buss multiplexor input

	--the signals that interconnect the hl and ll
	signal mcu_hl_ll_data_out 			: std_logic_vector(31 downto 0);
	signal mcu_hl_ll_data_in			: std_logic_vector(31 downto 0);
	signal mcu_hl_ll_address 			: std_logic_vector(31 downto 2);
	signal mcu_hl_ll_xfer_write		: std_logic;
	signal mcu_hl_ll_xfer_count		: std_logic_vector(11 downto 0);
	signal mcu_hl_ll_data_valid_done : std_logic;
	signal mcu_hl_ll_start_strobe		: std_logic;
	signal mcu_hl_ll_idle				: std_logic;

	--signal mcu_hl_busy : std_logic;

	component mcu_ll is
   Port (
		dram_d_buss					: inout std_logic_vector(31 downto 0);
		dram_a_buss 				: out std_logic_vector(11 downto 0);
		dram_ras_0 					: out std_logic;
		dram_ras_1 					: out std_logic;
		dram_ras_2 					: out std_logic;
		dram_ras_3 					: out std_logic;
		dram_cas_0 					: out std_logic;
		dram_cas_1 					: out std_logic;
		dram_cas_2 					: out std_logic;
		dram_cas_3 					: out std_logic;
		dram_we						: out std_logic;
		tvc_clock					: in std_logic;
		tvc_reset					: in std_logic;
		mcu_hl_data_in 			: in std_logic_vector(31 downto 0);
		mcu_hl_data_out			: out std_logic_vector(31 downto 0);
		mcu_hl_address				: in std_logic_vector(31 downto 2);
		mcu_hl_write_data			: in std_logic;
		mcu_hl_xfer_count			: in std_logic_vector(11 downto 0);
		mcu_hl_data_valid_done 	: out std_logic;
		mcu_hl_start_strobe 		: in std_logic;
		mcu_hl_idle					: out std_logic );
	end component;
	
begin

	--multiplexer for mu_bus signals that go to both the mcu_sched and the mcu_hl
	mu_bus_address <= mcu_hl_mu_bus_address	when mcu_sched_hl_busy = '1' else sched_mu_bus_address;

	--mu_bus_data <= mcu_hl_mu_bus_data;
	
	mu_bus_write <= mcu_hl_mu_bus_write when mcu_sched_hl_busy = '1' else sched_mu_bus_write;
	mu_bus_strobe	<= mcu_hl_mu_bus_strobe when mcu_sched_hl_busy = '1' else sched_mu_bus_strobe;


   sched : mcu_sched
	port map(tvc_clock=>tvc_clock,  tvc_reset=>tvc_reset,
				mu_bus_address=>sched_mu_bus_address, mu_bus_write=>sched_mu_bus_write, mu_bus_strobe=>sched_mu_bus_strobe,
				mcu_hl_start=>mcu_sched_hl_start, mcu_hl_busy=>mcu_sched_hl_busy,
				odd_sc_irq=>odd_sc_irq, odd_sc_select=>odd_sc_select, even_sc_irq=>even_sc_irq, even_sc_select=>even_sc_select,
				block_reader_irq=>block_reader_irq, block_reader_select=>block_reader_select,
				block_writer_irq=>block_writer_irq, block_writer_select=>block_writer_select,			 
				rp_writer_irq=>rp_writer_irq, rp_writer_select=>rp_writer_select,
				line_drawer_irq=>line_drawer_irq, line_drawer_select=>line_drawer_select,
				poly_scanline_irq=>poly_scanline_irq, poly_scanline_select=>poly_scanline_select);


	mcu_hl_comp : mcu_hl
   port map (mu_bus_address=>mcu_hl_mu_bus_address, mu_bus_data=>mu_bus_data, mu_bus_write=>mcu_hl_mu_bus_write, mu_bus_strobe=>mcu_hl_mu_bus_strobe,
				tvc_clock=>tvc_clock, tvc_reset=>tvc_reset, mcu_hl_start_strobe=>mcu_sched_hl_start, mcu_hl_busy=>mcu_sched_hl_busy,
				mcu_ll_data_out=>mcu_hl_ll_data_out, mcu_ll_data_in=>mcu_hl_ll_data_in,
				mcu_ll_address=>mcu_hl_ll_address, mcu_ll_xfer_write=>mcu_hl_ll_xfer_write, mcu_ll_xfer_count=>mcu_hl_ll_xfer_count,
				mcu_ll_data_valid_done=>mcu_hl_ll_data_valid_done, mcu_ll_start_strobe=>mcu_hl_ll_start_strobe, mcu_ll_idle=>mcu_hl_ll_idle);

	mcu_ll_comp : mcu_ll
   port map(dram_d_buss=>dram_d_buss, dram_a_buss=>dram_a_buss, dram_ras_0=>dram_ras_0, dram_ras_1=>dram_ras_1, dram_ras_2=>dram_ras_2, dram_ras_3=>dram_ras_3,
			dram_cas_0=>dram_cas_0, dram_cas_1=>dram_cas_1, dram_cas_2=>dram_cas_2, dram_cas_3=>dram_cas_3, dram_we=>dram_we,
			tvc_clock=>tvc_clock, tvc_reset=>tvc_reset,
			mcu_hl_data_in=>mcu_hl_ll_data_out, mcu_hl_data_out=>mcu_hl_ll_data_in,
			mcu_hl_address=>mcu_hl_ll_address, mcu_hl_write_data=>mcu_hl_ll_xfer_write, mcu_hl_xfer_count=>mcu_hl_ll_xfer_count,
			mcu_hl_data_valid_done=>mcu_hl_ll_data_valid_done, mcu_hl_start_strobe=>mcu_hl_ll_start_strobe, mcu_hl_idle=>mcu_hl_ll_idle);
	
end Behavioral;

