----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:25:24 08/05/2006 
-- Design Name: 
-- Module Name:    poly_scanline - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity poly_scanline is
   Port (
   --memory user access bus 
   mu_bus_address 	: in     std_logic_vector(11 downto 0);
   mu_bus_data		   : inout  std_logic_vector(31 downto 0);
   mu_bus_write		: in     std_logic; --if high the MCU is going to write to a mu
   mu_bus_strobe		: in     std_logic;

   poly_scanline_irq		: out std_logic;
   poly_scanline_select	: in  std_logic;

   tvc_clock		: in	std_logic;
   tvc_reset		: in 	std_logic;

   --interface to the eppcu
   cmd_bus_data         : in std_logic_vector(31 downto 0);
   cmd_bus_addr         : in std_logic_vector(7 downto 0);
   cmd_bus_strobe       : in std_logic;

   epp_idle					: out std_logic
   );
end poly_scanline;

architecture Behavioral of poly_scanline is

	TYPE PS_INT_STATE is ( PS_IDLE, PS_FETCH_FRAMEBUFFER, PS_FETCH_DEPTHBUFFER,
								PS_COMPUTE_1, PS_COMPUTE_1A, PS_COMPUTE_2,
                        PS_TEX_ADDRESS_1, PS_TEX_ADDRESS_2, PS_TEX_LOOKUP,
                        PS_COMPLETE_COMPUTE_LL,
								PS_WRITE_FRAMEBUFFER, PS_WRITE_DEPTHBUFFER,
								PS_COMPLETE_COMPUTE, PS_COMPLETE);
	signal ps_state : PS_INT_STATE := PS_IDLE;

	signal scanline_remaining	: std_logic_vector(15 downto 0) := (others => '0');

 	signal mu_bus_data_out : std_logic_vector(31 downto 0) := (others => '0');
	signal selected_once    : std_logic := '0';

	signal work_curr_num	   : std_logic_vector(4 downto 0) := (others => '0'); --the current pixel in the cache (used in compute)
	signal work_num_pixels  : std_logic_vector(4 downto 0) := (others => '0'); --how many pixels in the poly caches. extra bit is tacky but important
	signal work_fb_address  : std_logic_vector(31 downto 0) := (others => '0');
	signal work_db_address  : std_logic_vector(31 downto 0) := (others => '0');

	signal curr_z_depth  : std_logic_vector(31 downto 0) := (others => '0'); --a signed value
	signal z_increment	: std_logic_vector(31 downto 0) := (others => '0'); -- a signed value
	signal pix_value     : std_logic_vector(7 downto 0) := (others => '0');

   signal tc_x       :  std_logic_vector(15 downto 0) := (others => '0');
   signal tc_y       :  std_logic_vector(15 downto 0) := (others => '0');
   signal tc_x_inc   :  std_logic_vector(15 downto 0) := (others => '0');
   signal tc_y_inc   :  std_logic_vector(15 downto 0) := (others => '0');
   signal texture_id :  std_logic_vector(2 downto 0) := (others => '0');

	component poly_cache is
	generic( WIDTH: positive );
	Port(
	addr 				: in std_logic_vector(3 downto 0);
	data_out 		: out std_logic_vector(WIDTH-1 downto 0);
	data_in			: in std_logic_vector(WIDTH-1 downto 0);
	write_enable 	: in std_logic;
	clk				: in std_logic
	);
	end component;
	
	signal fbc_data_out 	: std_logic_vector(31 downto 0) := (others => '0');
	signal fbc_data_in	: std_logic_vector(31 downto 0) := (others => '0');
	signal fbc_addr 		: std_logic_vector(3 downto 0) := (others => '0');
	signal fbc_write		: std_logic := '0';

	signal dc_data_out	: std_logic_vector(31 downto 0) := (others => '0');
	signal dc_data_in	 	: std_logic_vector(31 downto 0) := (others => '0');
	signal dc_addr 		: std_logic_vector(3 downto 0) := (others => '0');
	signal dc_write 		: std_logic := '0';

   signal mult_bit_mask : std_logic_vector(7 downto 0);
   signal mult_n1_full  : std_logic_vector(15 downto 0);
   signal tex_address   : std_logic_vector(23 downto 0); --should be 32, but running out of space in fpga..

	signal inc_address      : std_logic := '0';
   signal depth_test_valid : std_logic := '0'; --we should update the front buffer and depth buffer
begin
	fb_cache : poly_cache generic map (WIDTH=>32)
		port map(addr=>fbc_addr, data_out=>fbc_data_in, data_in=>fbc_data_out, write_enable=>fbc_write, clk=>tvc_clock);

	depth_cache : poly_cache generic map (WIDTH=>32)
		port map(addr=>dc_addr, data_out=>dc_data_in, data_in=>dc_data_out, write_enable=>dc_write, clk=>tvc_clock);

	process(ps_state)
	begin
		if ps_state = PS_IDLE then
			epp_idle <= '1';
		else
			epp_idle <= '0';
		end if;
	end process;

	process(mu_bus_data_out, poly_scanline_select, mu_bus_strobe, mu_bus_write)
	begin
		if poly_scanline_select = '1' and mu_bus_strobe = '1' and mu_bus_write = '0' then
			mu_bus_data <= mu_bus_data_out;
		else
			mu_bus_data <= (others => 'Z');
		end if;
	end process;

	Process (tvc_clock)
		variable do_fbc_write : boolean;
		variable do_dc_write	: boolean;
	begin
	if tvc_clock = '1' and tvc_clock'event then
	if tvc_reset = '1' then
		ps_state <= PS_IDLE;
		mu_bus_data_out <= (others => '0');
		
		--mu_bus_data --handled in process
		poly_scanline_irq <= '0';
		--epp_idle --handled in process
	else --not reset
		do_fbc_write 	:= false;
		do_dc_write		:= false;
		
		case ps_state is
		when PS_IDLE =>
			if cmd_bus_strobe = '1' then
            case cmd_bus_addr is
            when X"F0" =>
               work_fb_address <= cmd_bus_data;
            when X"F1" =>
               work_db_address <= cmd_bus_data;
            when X"F2" =>
               curr_z_depth <= cmd_bus_data;
            when X"F3" =>
               z_increment <= cmd_bus_data;
            when X"F4" =>
               scanline_remaining <= cmd_bus_data(15 downto 0);
            when X"F5" =>
               pix_value <= cmd_bus_data(7 downto 0);
            when X"F6" =>
               tc_x <= cmd_bus_data(31 downto 16);
               tc_y <= cmd_bus_data(15 downto 0);
            when X"F7" =>
               tc_x_inc <= cmd_bus_data(31 downto 16);
               tc_y_inc <= cmd_bus_data(15 downto 0);
            when X"F8" =>
               texture_id <= cmd_bus_data(2 downto 0);
            when X"F9" => --note we don't care about command bus value
               if scanline_remaining > X"000F" then
                  work_num_pixels <= "1" & X"0";
               else
                  work_num_pixels <= "0" & scanline_remaining(3 downto 0);
               end if;
               
               if scanline_remaining /= X"0000" then
                  ps_state <= PS_FETCH_FRAMEBUFFER;
                  --ps_state <= PS_FETCH_DEPTHBUFFER;
                  poly_scanline_irq <= '1';
                  selected_once <= '0';
               end if;
            when others =>
               null;
            end case;
         end if;
		when PS_FETCH_FRAMEBUFFER =>
			if selected_once = '1' and poly_scanline_select = '0' then
				ps_state <= PS_FETCH_DEPTHBUFFER;
				selected_once <= '0';
              --note that we leave the irq line high.
			elsif poly_scanline_select = '1' then
				selected_once <= '1';
				if mu_bus_write = '1' and mu_bus_strobe = '1' then
					fbc_data_out <= mu_bus_data;
					fbc_addr <= mu_bus_address(3 downto 0);
					do_fbc_write := true;
				else
					if mu_bus_write = '0' and mu_bus_strobe = '1' then
					case mu_bus_address is
					when X"FFC" =>
						mu_bus_data_out <= X"00" & X"00_00" & "000" & work_num_pixels;
					when X"FFD" =>
						mu_bus_data_out <= work_fb_address;
					when others =>
						mu_bus_data_out <= (others => '0');
					end case;
					end if;				
				end if;
			end if;
		when PS_FETCH_DEPTHBUFFER =>
			if selected_once = '1' and poly_scanline_select = '0' then
				ps_state <= PS_COMPUTE_1;
				work_curr_num <= "0" & X"0";
				selected_once <= '0';
				poly_scanline_irq <= '0';
			elsif poly_scanline_select = '1' then
				selected_once <= '1';
				if mu_bus_write = '1' and mu_bus_strobe = '1' then
					dc_data_out <= mu_bus_data;
					dc_addr <= mu_bus_address(3 downto 0);
					do_dc_write := true;
				else
					if mu_bus_write = '0' and mu_bus_strobe = '1' then
						case mu_bus_address is
						when X"FFC" =>
							mu_bus_data_out <= X"00" & X"0000" & "0" & work_num_pixels & "00";
						when X"FFD" =>
							mu_bus_data_out <= work_db_address;
						when others =>
							mu_bus_data_out <= (others => '0');
						end case;
					end if;
				end if;
			end if;
		when PS_COMPUTE_1 =>
			fbc_addr <= "00" & work_curr_num(3 downto 2);
			dc_addr <= work_curr_num(3 downto 0);
 
         case texture_id is
            when "001" =>
               tex_address <= X"30_0000";
            when "010" =>
               tex_address <= X"38_0000";
            when "011" =>
               tex_address <= X"40_0000";
            when "100" =>
               tex_address <= X"48_0000";
            when "101" =>
               tex_address <= X"50_0000";
            when "110" =>
               tex_address <= X"58_0000";
            when "111" =>
               tex_address <= X"60_0000";
            when others =>
               null;
         end case;
         
         ps_state <= PS_COMPUTE_1A;

       when PS_COMPUTE_1A => --this state is a hack.
          if signed( curr_z_depth ) > signed( dc_data_in ) then
            depth_test_valid  <= '1';
         else
            depth_test_valid <= '0';
         end if;
         
			if texture_id /= "000" then
            ps_state <= PS_TEX_ADDRESS_1;
            mult_bit_mask <= "00000001";
            mult_n1_full <= X"01" & X"00";
         else
            --tex_address <= (others => '0');
            ps_state <= PS_COMPUTE_2;  
         end if;       
      when PS_TEX_ADDRESS_1 =>
         if ( tc_y(15 downto 8) AND mult_bit_mask ) = mult_bit_mask then
            tex_address <= tex_address + ( mult_n1_full & "00");
         end if;
         mult_n1_full <= mult_n1_full(14 downto 0) & '0'; --left shift
         mult_bit_mask <= mult_bit_mask(6 downto 0) & '0'; --left shift
         if depth_test_valid = '0' then --bail out of texture address calculations.
            pix_value <= X"00";
            ps_state <= PS_COMPUTE_2;
         elsif mult_bit_mask(7) = '1' then --final bit
            ps_state <= PS_TEX_ADDRESS_2;
         end if;
      when PS_TEX_ADDRESS_2 =>
         tex_address <= tex_address + (tc_x(15 downto 8) & "00");
         ps_state <= PS_TEX_LOOKUP;
         poly_scanline_irq <= '1';
      when PS_TEX_LOOKUP =>
			if selected_once = '1' and poly_scanline_select = '0' then
				ps_state <= PS_COMPUTE_2;
				selected_once <= '0';
				poly_scanline_irq <= '0';
			elsif poly_scanline_select = '1' then
				selected_once <= '1';
				if mu_bus_write = '1' and mu_bus_strobe = '1' then
					pix_value <= mu_bus_data(31 downto 30) & mu_bus_data(23 downto 21) & mu_bus_data(15 downto 13);
				else
					if mu_bus_write = '0' and mu_bus_strobe = '1' then
						case mu_bus_address is
						when X"FFC" =>
							mu_bus_data_out <= X"0000000" & "0100"; --fetch one 32 bit value
						when X"FFD" =>
							mu_bus_data_out <= X"00" & tex_address;
						when others =>
							mu_bus_data_out <= (others => '0');
						end case;
					end if;
				end if;
			end if;      
         when PS_COMPUTE_2 =>
			if depth_test_valid = '1' then
				case work_curr_num(1 downto 0) is
				when "00" =>
					fbc_data_out <= pix_value & fbc_data_in(23 downto 0);
				when "01" =>
					fbc_data_out <= fbc_data_in(31 downto 24) & pix_value & fbc_data_in(15 downto 0);
				when "10" =>
					fbc_data_out <= fbc_data_in(31 downto 16) & pix_value & fbc_data_in(7 downto 0);
				when others => --'11'
					fbc_data_out <= fbc_data_in(31 downto 8) & pix_value;
				end case;
				
				dc_data_out <= curr_z_depth;
				do_fbc_write := true;
				do_dc_write := true;
			end if;
         
         --fbc_data_out <= X"FFFF_FFFF";
         --do_fbc_write := true;
            
			curr_z_depth <= signed(curr_z_depth) + signed(z_increment);
         if texture_id /= "000" then
            tc_x <= signed(tc_x) + signed(tc_x_inc);
            tc_y <= signed(tc_y) + signed(tc_y_inc);
         end if;
         
			if work_curr_num = work_num_pixels then
				ps_state <= PS_COMPLETE_COMPUTE_LL;
			else
				work_curr_num <= work_curr_num + 1;
				ps_state <= PS_COMPUTE_1;
			end if;
		when PS_COMPLETE_COMPUTE_LL =>
			ps_state <= PS_WRITE_FRAMEBUFFER;
			poly_scanline_irq <= '1';
			selected_once <= '0';
			inc_address <= '0';
			fbc_addr <= "0000";
			dc_addr <= "0000";
		when PS_WRITE_FRAMEBUFFER=>
			if selected_once = '1' and poly_scanline_select = '0' then
				ps_state <= PS_WRITE_DEPTHBUFFER;
				--poly_scanline_irq <= '1'; leave it set from write framebuffer.
				selected_once <= '0';
				inc_address <= '0';
				dc_addr <= "0000";
			elsif poly_scanline_select = '1' then
				selected_once <= '1';
				if mu_bus_write = '0' then
					if mu_bus_strobe = '1' then
						case mu_bus_address is
						when X"FFC" =>
							mu_bus_data_out <= X"80" & X"00_00" & "000" & work_num_pixels;
						when X"FFD" =>
							mu_bus_data_out <= work_fb_address;
						when others => --note our disregarding the mu_bus_address.
							mu_bus_data_out <= fbc_data_in;
							--mu_bus_data_out <= X"FFFF_FFFF";
							inc_address <= '1';
						end case;
					else
						if inc_address = '1' then
							fbc_addr <= fbc_addr + 1;
							inc_address <= '0';
						end if;				
					end if;
				end if;
			end if;
		when PS_WRITE_DEPTHBUFFER=>
			if selected_once = '1' and poly_scanline_select = '0' then
				ps_state <= PS_COMPLETE_COMPUTE;
				poly_scanline_irq <= '0';
			elsif poly_scanline_select = '1' then
				selected_once <= '1';
				if mu_bus_write = '0' then
					if mu_bus_strobe = '1' then
						case mu_bus_address is
						when X"FFC" =>
							mu_bus_data_out <= X"80" & X"00_00" & "0" & work_num_pixels & "00";
						when X"FFD" =>
							mu_bus_data_out <= work_db_address;
						when others => --note our disregarding the mu_bus_address.
							mu_bus_data_out <= dc_data_in;
							inc_address <= '1';
						end case;
					else
						if inc_address = '1' then
							dc_addr <= dc_addr + 1;
							inc_address <= '0';
						end if;				
					end if;
				end if;
			end if;
		when PS_COMPLETE_COMPUTE => --should be pushed into WRITE_DEPTHBUFFER
			scanline_remaining <= scanline_remaining - work_num_pixels;
			work_fb_address <= work_fb_address + work_num_pixels;
			work_db_address <= work_db_address + ( work_num_pixels & "00" );
			ps_state <= PS_COMPLETE;
		when PS_COMPLETE =>
			if scanline_remaining = X"0000" then
				ps_state <= PS_IDLE;
			else
				if scanline_remaining > X"000F" then
					work_num_pixels <= "1" & X"0";
				else
					work_num_pixels <= "0" & scanline_remaining(3 downto 0);
				end if;
				ps_state <= PS_FETCH_FRAMEBUFFER;
				--ps_state <= PS_FETCH_DEPTHBUFFER;
				poly_scanline_irq <= '1';
				selected_once <= '0';
			end if;
		end case;
		
		if do_fbc_write = true then
			fbc_write <= '1';
		else
			fbc_write <= '0';
		end if;
		
		if do_dc_write = true then
			dc_write <= '1';
		else
			dc_write <= '0';
		end if;
		
	end if;
	end if; --clock
	end Process;
	
end Behavioral;

