library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;
use UNISIM.VComponents.all;

--library work;
--use work.all;

entity tvc is		    
   Port (		 
   tvc_clock 				: in 	std_logic;
   tvc_reset_in 			: in 	std_logic;

   led_out					: out	std_logic;

   epp_data 				: inout	std_logic_vector(7 downto 0);
   epp_write 				: in 	std_logic;
   epp_wait 				: out 	std_logic;
   epp_data_strobe 		: in 	std_logic;
   epp_address_strobe 	: in 	std_logic;
   epp_irq					: out	std_logic;
   --         epp_reset 		: in 	std_logic;

   disp1_pix_value 		: out 	std_logic_vector(7 downto 0);
   disp1_horiz_sync 		: out 	std_logic;
   disp1_vert_sync 		: out 	std_logic;

   dram_d_buss				: inout std_logic_vector(31 downto 0);
   dram_a_buss				: out std_logic_vector(11 downto 0);
   dram_ras_0				: out std_logic;
   dram_ras_1				: out std_logic;
   dram_ras_2				: out std_logic;
   dram_ras_3				: out std_logic;
   dram_cas_0				: out std_logic;
   dram_cas_1				: out std_logic;
   dram_cas_2				: out std_logic;
   dram_cas_3				: out std_logic;
   dram_we					: out std_logic
   );
end tvc;

architecture Behavioral of tvc is
	component reset_sync
	Port (
	sig_in	: in	STD_LOGIC;
	sig_out	: out	STD_LOGIC;
	clock		: in	STD_LOGIC
	);
	end component;

	component pix_pusher
	Port(
	mu_bus_address 	: in std_logic_vector(11 downto 0);
	mu_bus_data		: inout std_logic_vector(31 downto 0);
	mu_bus_write		: in std_logic;
	mu_bus_strobe		: in std_logic;
	odd_sc_irq			: out std_logic;
	odd_sc_select		: in std_logic;
	even_sc_irq			: out std_logic;
	even_sc_select		: in std_logic;
	tvc_clock			: in 	std_logic;
	tvc_reset			: in 	std_logic;
	disp_pix_value 	: out 	std_logic_vector(7 downto 0);
	disp_horiz_sync 	: out 	std_logic;
	disp_vert_sync 	: out 	std_logic
	);
	end component;

	component eppcu
	Port(
	epp_data             : inout std_logic_vector(7 downto 0);
	epp_write            : in 	std_logic;
	epp_wait             : out std_logic;
	epp_data_strobe      : in 	std_logic;
	epp_address_strobe   : in 	std_logic;
	epp_irq              : out	std_logic;
	tvc_clock				: in std_logic;
	tvc_reset				: in std_logic;
	cmd_bus_data         : out std_logic_vector(31 downto 0);
   cmd_bus_addr         : out std_logic_vector(7 downto 0);
   cmd_bus_strobe       : out std_logic;
   cmd_bus_data_out     : in std_logic_vector(31 downto 0); --but data into the eppcu
	br_idle					: in std_logic;
	bw_idle					: in std_logic;
	bs_idle					: in std_logic;
 	ps_idle					: in  std_logic
	);
	end component;

	component mcu_struct is
    Port (
   dram_d_buss		: inout std_logic_vector(31 downto 0);
   dram_a_buss 	: out std_logic_vector(11 downto 0);
   dram_ras_0 		: out std_logic;
   dram_ras_1 		: out std_logic;
   dram_ras_2 		: out std_logic;
   dram_ras_3 		: out std_logic;
   dram_cas_0 		: out std_logic;
   dram_cas_1 		: out std_logic;
   dram_cas_2 		: out std_logic;
   dram_cas_3 		: out std_logic;
   dram_we			: out std_logic;
   tvc_clock		: in	std_logic;
   tvc_reset		: in 	std_logic;
   mu_bus_address 	: out std_logic_vector(11 downto 0);
   mu_bus_data		: inout std_logic_vector(31 downto 0);
   mu_bus_write		: out  std_logic;
   mu_bus_strobe			: out  std_logic;
   odd_sc_irq				: in std_logic;
   odd_sc_select			: out std_logic;
   even_sc_irq				: in std_logic;
   even_sc_select			: out std_logic;
   block_reader_irq		: in std_logic;
   block_reader_select	: out std_logic;
   block_writer_irq		: in std_logic;
   block_writer_select	: out std_logic;				 
   rp_writer_irq			: in std_logic;
   rp_writer_select		: out std_logic;
   line_drawer_irq		: in std_logic;
   line_drawer_select	: out std_logic;
   poly_scanline_irq		: in std_logic;
   poly_scanline_select : out std_logic
   );	
	end component;
 
	component block_reader is
	Port(
	mu_bus_address       : in std_logic_vector(11 downto 0);
	mu_bus_data          : inout std_logic_vector(31 downto 0);
	mu_bus_write         : in std_logic; --if high the MCU is going to write to a mu
	mu_bus_strobe			: in std_logic;
	block_reader_irq		: out std_logic;
	block_reader_select	: in std_logic;
	tvc_clock            : in	std_logic;
	tvc_reset            : in std_logic;
   cmd_bus_data         : in std_logic_vector(31 downto 0);
   cmd_bus_addr         : in std_logic_vector(7 downto 0);
   cmd_bus_strobe       : in std_logic;
   cmd_bus_data_out     : out std_logic_vector(31 downto 0);
	epp_idle             : out std_logic
	);
	end component;

	component block_writer is
	Port(
	mu_bus_address 	   : in std_logic_vector(11 downto 0);
	mu_bus_data		      : inout std_logic_vector(31 downto 0);
	mu_bus_write		   : in  std_logic;
	mu_bus_strobe		   : in  std_logic;
	block_writer_irq		: out std_logic;
	block_writer_select	: in std_logic;
	tvc_clock			   : in	std_logic;
	tvc_reset			   : in 	std_logic;
   cmd_bus_data         : in std_logic_vector(31 downto 0);
   cmd_bus_addr         : in std_logic_vector(7 downto 0);
   cmd_bus_strobe       : in std_logic;
	epp_idle			      : out std_logic
	);
	end component;

	component block_set is
	Port(
	mu_bus_address 	: in std_logic_vector(11 downto 0);
	mu_bus_data		: inout std_logic_vector(31 downto 0);
	mu_bus_write		: in  std_logic;
	mu_bus_strobe		: in  std_logic;
	block_set_irq		: out std_logic;
	block_set_select	: in std_logic;
	tvc_clock			: in std_logic;
	tvc_reset			: in std_logic;
   cmd_bus_data      : in std_logic_vector(31 downto 0);
   cmd_bus_addr      : in std_logic_vector(7 downto 0);
   cmd_bus_strobe    : in std_logic;
	epp_idle				: out std_logic
	);
	end component;

	component poly_scanline is
   Port (
	mu_bus_address 		: in std_logic_vector(11 downto 0);
	mu_bus_data			: inout std_logic_vector(31 downto 0);
	mu_bus_write			: in  std_logic;
	mu_bus_strobe			: in  std_logic;
	poly_scanline_irq		: out std_logic;
	poly_scanline_select	: in std_logic;
	tvc_clock				: in	std_logic;
	tvc_reset				: in 	std_logic;
   cmd_bus_data         : in std_logic_vector(31 downto 0);
   cmd_bus_addr         : in std_logic_vector(7 downto 0);
   cmd_bus_strobe       : in std_logic;
   epp_idle					: out std_logic
	);
	end component;

--	component IBUFG
--	port (
--		 I : in STD_LOGIC;
--		 O : out STD_LOGIC);
--	end component;

	signal tvc_reset				: std_logic;
--start mcu related signals
	signal mu_bus_address		: std_logic_vector(11 downto 0);
	signal mu_bus_data			: std_logic_vector(31 downto 0);
	signal mu_bus_write			: std_logic;
	signal mu_bus_strobe		: std_logic;

	signal odd_sc_irq				: std_logic;
	signal odd_sc_select			: std_logic;
	signal even_sc_irq			: std_logic;
	signal even_sc_select		: std_logic;
	signal block_reader_irq		: std_logic;
	signal block_reader_select	: std_logic;
	signal block_writer_irq		: std_logic;
	signal block_writer_select	: std_logic;
	signal block_set_irq			: std_logic;
	signal block_set_select		: std_logic;
	signal line_drawer_irq		: std_logic;
	signal line_drawer_select	: std_logic;
	signal poly_scanline_irq	: std_logic;
	signal poly_scanline_select : std_logic;

--begin eppcu command bus signals
	signal cmd_bus_data           : std_logic_vector(31 downto 0);
   signal cmd_bus_addr           : std_logic_vector(7 downto 0);
   signal cmd_bus_strobe         : std_logic;
   signal cmd_bus_data_out       : std_logic_vector(31 downto 0);

--begin eppcu and block_reader interface logic
	signal br_epp_idle			: std_logic;

--begin eppcu and block_writer interface logic
	signal bw_epp_idle			: std_logic;

--begin eppcu and block set
	signal bs_epp_idle				: std_logic;

--begin eppcu and rp_writer
--	signal rpw_epp_idle				: std_logic;

--begin eppcu and poly_scanline mu
	signal ps_epp_idle				: std_logic;

begin
  	--tvc_reset <= NOT tvc_reset_in;
	led_out <= tvc_reset;
	--led_out <= '0';
	
	line_drawer_irq <= '0';

	rsync : reset_sync
	port map( sig_in=> NOT tvc_reset_in, sig_out=>tvc_reset, clock=>tvc_clock);
	
	ppusher : pix_pusher
	port map(mu_bus_address=>mu_bus_address, mu_bus_data=>mu_bus_data, mu_bus_write=>mu_bus_write, mu_bus_strobe=>mu_bus_strobe,
				odd_sc_irq=>odd_sc_irq, odd_sc_select=>odd_sc_select, even_sc_irq=>even_sc_irq, even_sc_select=>even_sc_select,
				tvc_clock=>tvc_clock,  tvc_reset=>tvc_reset, disp_pix_value=>disp1_pix_value,
		    	disp_horiz_sync=>disp1_horiz_sync, disp_vert_sync=>disp1_vert_sync);

	control_unit : eppcu
		port map(epp_data=>epp_data, epp_write=>epp_write, epp_wait=>epp_wait, epp_data_strobe=>epp_data_strobe,
				epp_address_strobe=>epp_address_strobe, epp_irq=>epp_irq, tvc_clock=>tvc_clock, tvc_reset=>tvc_reset,
            cmd_bus_data=>cmd_bus_data, cmd_bus_addr=>cmd_bus_addr, cmd_bus_strobe=>cmd_bus_strobe, cmd_bus_data_out=>cmd_bus_data_out,
				br_idle=>br_epp_idle,
				bw_idle=>bw_epp_idle,
				bs_idle=>bs_epp_idle,
            ps_idle=>ps_epp_idle);

	mem_control_unit : mcu_struct
		port map(	dram_d_buss=>dram_d_buss, dram_a_buss=>dram_a_buss,
				dram_ras_0=>dram_ras_0, dram_ras_1=>dram_ras_1, dram_ras_2=>dram_ras_2, dram_ras_3=>dram_ras_3,
				dram_cas_0=>dram_cas_0, dram_cas_1=>dram_cas_1, dram_cas_2=>dram_cas_2, dram_cas_3=>dram_cas_3,
				dram_we=>dram_we,
				tvc_clock=>tvc_clock, tvc_reset=>tvc_reset,
				mu_bus_address=>mu_bus_address, mu_bus_data=>mu_bus_data, mu_bus_write=>mu_bus_write, mu_bus_strobe=>mu_bus_strobe,
				odd_sc_irq=>odd_sc_irq, odd_sc_select=>odd_sc_select, even_sc_irq=>even_sc_irq, even_sc_select=>even_sc_select,
				block_reader_irq=>block_reader_irq, block_reader_select=>block_reader_select,
				block_writer_irq=>block_writer_irq, block_writer_select=>block_writer_select,
				rp_writer_irq=>block_set_irq, rp_writer_select=>block_set_select,
				line_drawer_irq=>line_drawer_irq, line_drawer_select=>line_drawer_select,
				poly_scanline_irq=>poly_scanline_irq, poly_scanline_select=>poly_scanline_select);

	--start memory users

	br_mu : block_reader
		port map(mu_bus_address=>mu_bus_address, mu_bus_data=>mu_bus_data, mu_bus_write=>mu_bus_write, mu_bus_strobe=>mu_bus_strobe,
				block_reader_irq=>block_reader_irq, block_reader_select=>block_reader_select,
				tvc_clock=>tvc_clock, tvc_reset=>tvc_reset,
            cmd_bus_data=>cmd_bus_data, cmd_bus_addr=>cmd_bus_addr, cmd_bus_strobe=>cmd_bus_strobe, cmd_bus_data_out=>cmd_bus_data_out,
				epp_idle=>br_epp_idle );

	bw_mu : block_writer
		port map(mu_bus_address=>mu_bus_address, mu_bus_data=>mu_bus_data, mu_bus_write=>mu_bus_write, mu_bus_strobe=>mu_bus_strobe,
				block_writer_irq=>block_writer_irq, block_writer_select=>block_writer_select,
				tvc_clock=>tvc_clock, tvc_reset=>tvc_reset,
            cmd_bus_data=>cmd_bus_data, cmd_bus_addr=>cmd_bus_addr, cmd_bus_strobe=>cmd_bus_strobe,
				epp_idle=>bw_epp_idle );

	bs_mu : block_set
		port map(mu_bus_address=>mu_bus_address, mu_bus_data=>mu_bus_data, mu_bus_write=>mu_bus_write, mu_bus_strobe=>mu_bus_strobe,
				block_set_irq=>block_set_irq, block_set_select=>block_set_select,
				tvc_clock=>tvc_clock, tvc_reset=>tvc_reset,
            cmd_bus_data=>cmd_bus_data, cmd_bus_addr=>cmd_bus_addr, cmd_bus_strobe=>cmd_bus_strobe,
				epp_idle=>bs_epp_idle);

	ps_mu : poly_scanline
		port map( mu_bus_address=>mu_bus_address, mu_bus_data=>mu_bus_data, mu_bus_write=>mu_bus_write, mu_bus_strobe=>mu_bus_strobe,		
				poly_scanline_irq=>poly_scanline_irq, poly_scanline_select=>poly_scanline_select, tvc_clock=>tvc_clock, tvc_reset=>tvc_reset,
            cmd_bus_data=>cmd_bus_data, cmd_bus_addr=>cmd_bus_addr, cmd_bus_strobe=>cmd_bus_strobe,
            epp_idle=>ps_epp_idle);

 
end Behavioral;
